/* aqv23tst8d.C - as aqv23tst8b.C but now does the case A test
   Calc 10.2
*/

#include "mydefs.h"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>


#define rint register int
#define cint const int

#define ERROR(a) if(a) { cerr<<"error line "<<__LINE__<<endl; }

inline int abs_member(const int a,const int *b,const u_int j)
{ for(u_int i=0;i<j;) if(abs(b[i++])==a) return i; return 0; }

struct edge {
  int g[2];
  int & oper [] (cint i) { ret g[i]; }
  const int oper [] (cint i) const { ret g[i]; }
  const edge & oper =(const edge & e) { g[1]=e[1]; g[0]=e[0]; ret e; }
  friend int vnr(int v,const edge & e) {
    ret e[0]==v ? 1 : e[1]==v ? 2:0; 
  }
  oper int * () { ret g; }
};

struct graph { // graph
  int nv,ne; // nr vert & edges
  int *verts;
  edge *edges;

  // edges[j][0]...[1] are the source & target verts of
  // ori edge #j (j=0,...,ne-1)
  // a vertex may be negative if it is marked (it's alw a non-0
  // int); even if marked, it does not appear signed in edges[][],
  // though

  ~graph() { delete [] verts; delete [] edges; }

  graph(const graph & g): nv(g.nv), ne(g.ne) {
    verts=copyarray(g.verts,nv);
    edges=copyarray(g.edges,ne);
  }

  const graph & oper =(const graph & g) { 
    delete [] verts; delete [] edges;
    nv=g.nv; ne=g.ne; verts=copyarray(g.verts,nv);     
    edges=copyarray(g.edges,ne);
    ret g;
  }

  graph(int nvr, int ned, int *v=0, int *e=0)
  {
    verts=new int[nv=nvr]; 
    edges=new edge[ne=ned];
    if(v) {
      copyarray(v,nv,verts);
      for(int j=0;j<ne;++j) copyarray(e+2*j,2,(int *)edges[j]);
    }
  }


  friend istream & oper >>(istream &in, graph & g)
  {
    int i,j;
    
    in>>g.nv>>g.ne;
    
    g=graph(g.nv,g.ne);

    for(i=0;i<g.nv;in>>g.verts[i++]);
    
    for(i=0;i<g.ne;++i) for(j=0;j<2;++j) in>>g.edges[i][j];

    ret in;
  }

  friend ostream & oper <<(ostream &o, const graph & g)
  {
    int i,j;
    
    varoutputarray(o<<"[ "<<g.nv,g.verts,g.nv," ("," ",")  |  ");
    o.flush();
    o<<g.ne;
    o.flush();

    for(i=0;i<g.ne;++i) {
      varoutputarray(o,(int *)g.edges[i],2," ("," ",")");
      o.flush();
    }

    ret o<<"  ] ";
  }

  void reduce() // remove loops
  {
    int j;

    for(j=0;j<ne;j++)
    {
      if(edges[j][0]==edges[j][1]) edges[j--]=edges[--ne];
    }
  }

  graph del(int nre) const
  { // delete edge #nre

    graph g=*this;

    g.edges[nre]=g.edges[--g.ne];

    ret g;
  }

  graph propagate() const
  {
    graph g=*this;

    int fl,j,k,l;

    do {

      fl=0;

      for(j=0;j<ne;j++) for(k=0;k<2;++k)

        if((l=member(g.edges[j][k],g.verts,g.nv)) &&
	   ! member(g.edges[j][1-k],g.verts,g.nv)
	  ) 
	  g.verts[l-1]*=-1,fl=1;
    }
    while(fl);

    ret g;
  }

  friend bool full_marked(const graph & g)
  {
    int j;

    for(j=0;j<g.nv && g.verts[j]<0;++j); ret j==g.nv;
  }

  graph mark(int nrv) const
  {
    graph g=*this;

    int j;

    for(j=0;j<g.nv;++j) {
      g.verts[j]=abs(g.verts[j])*(nrv==j?-1:1);
      if(!g.verts[j]) cerr<<"error vert #0 found"<<endl;
    }

    ret g;
  }

  graph add_mark(int vr) const
  {
    // mark vertex named (NOT #ed) vr;

    graph g=*this;

    if(int m=member(vr,g.verts,g.nv)) g.verts[m-1]=-abs(g.verts[m-1]);

    ret g;
  }

  bool isol_vert(int nrv) const
  {
    graph g=mark(nrv);
    ret cmparray(g.verts,g.propagate().verts,g.nv);
  }

  friend bool connd(const graph & g, int *cmp=0) 
  // is g conn'd?
  // if not one can store in cmp[] the markings of vertices
  // of diff comps of g (needed for block component determination
  // in index() )
  {
    graph h=g.mark(0).propagate(); 
    if(cmp) copyarray(h.verts,h.nv,cmp);
    ret full_marked(h);
  }

  bool parallel(int e1, int e2) const {
    // are edges e1, e2 parallel (i.e. form a multiple edge)
    ret edges[e1][0]==edges[e2][0] && edges[e1][1]==edges[e2][1] ||
        edges[e1][1]==edges[e2][0] && edges[e1][0]==edges[e2][1];
  }

  bool adj(int v1, int v2) const {
    // are verts v1,v2 adjacent (i.e. conn'd by an edge)
    for(int i=0;i<ne;++i)
    if(edges[i][0]==v1 && edges[i][1]==v2 ||
       edges[i][0]==v2 && edges[i][1]==v1) ret true;
    ret false;
  }

  bool simple(int enr) const {
    // is edge #enr a simple edge ?
    for(int j=0;j<ne;++j) if(j-enr && parallel(j,enr)) ret false;
    ret true;
  }

  graph cut(int vrt) const { 
    // delete the star of vertex vrt& vert itself
    // vrt must be negative if vertex -vet is marked in *this !
    int nrv=member(vrt,verts,nv)-1,j;
    if(nrv<0) {
      cerr<<" try to del non-exist vrt"<<endl; ret *this;
    }
    graph g=*this;

    for(j=0;j<g.ne;++j) {
      if(vnr(abs(vrt),g.edges[j]))
	// abs() to account for marked vertex
	g.edges[j--]=g.edges[--g.ne];
    }

    g.verts[nrv]=g.verts[--g.nv];

    // TRC(g);
    ret g;
  }
  
  bool isepar() const { // no cut vert?
    // TRC(*this);
    for(int j=0;j<nv;++j) {
      if(!connd(cut(verts[j]))) ret false;
    }
    
    ret connd(*this);
  }

  int atomnr() const { // Mur-Prz index until you reach mx

    if(nv<3) ret 1;
    // TRC(*this);
    int i,j,z,y,x[nv],gi,gj,k;

    // 1st try block sum decomp; but must ensure that cut vertex 
    // is not adjacent to marked edge

    for(j=0;j<nv;++j) {
      if(!connd(cut(y=verts[j]),x)) {      
	graph g=*this, g1=g;
	for(i=0;i<nv-1;++i) {
	  graph & y= x[i]>0 ? g1:g;
	  k=abs_member(abs(x[i]),y.verts,y.nv)-1;
	  ERROR(k<0);
	  y=y.cut(y.verts[k]);
	}
	ret g.atomnr()+g1.atomnr();
      }
    }

    ret 1;
  }
};

class tracedg {
public:
  int ncr, nl, *len, *conn;

  ~tracedg() { delete [] len; delete [] conn; }

  tracedg(const tracedg & g): ncr(g.ncr), nl(g.nl) {
    len=copyarray(g.len,nl);
    conn=copyarray(g.conn,nl*ncr);
  }

  const tracedg & oper =(const tracedg & g) { 
    delete [] len; delete [] conn;
    ncr=g.ncr; len=copyarray(g.len,nl=g.nl);     
    conn=copyarray(g.conn,ncr*ncr);
    ret g;
  }

  tracedg(cint nc=0, cint nlp=0,  cint *l=0, cint **c=0)
  {
    // TRC(nc);
    ncr=nc;
    len=new int[nl=nlp]; 
    conn=new int[nl*nc];
    // TRC(l);
    if(l) {
      copyarray(l,nl,len);
      for(int j=0;j<nl;++j) {
	// TRC(j);
	copyarray(c[j],l[j],conn+nc*j);
      }
    }
    // TRC(*this);
  }


  friend istream & oper >>(istream &in, tracedg & g)
  {
    int i,j;
    // in must contain:
    // #crsgs, #loops, array of lengths (#of trace basepts) for
    // loops, arrays of basepts (numbered from 1)
    
    in>>g.ncr>>g.nl;
    
    g=tracedg(g.ncr,g.nl);

    for(i=0;i<g.nl;in>>g.len[i++]);
    
    for(i=0;i<g.nl;++i) for(j=0;j<g.len[i];++j)
      in>>g.conn[i*g.ncr+j];

    ret in;
  }

  friend ostream & oper <<(ostream &o, const tracedg & g)
  {
    int i;
    
    varoutputarray(o<<"[ "<<g.ncr<<" "<<g.nl,
      g.len,g.nl," ("," ",")  |  ");

    for(i=0;i<g.nl;++i) {
      varoutputarray(
	o,g.conn+i*g.ncr, g.len[i]," ("," ",") "
      );
      o.flush();
    }

    ret o<<"  ] ";
  }

  tracedg AtoB (int &nnq) const;
  tracedg del(int e) const;
  tracedg mult(int e, int mult) const;
  int nrcomp() const;
  tracedg rtwist(int e) const;
  void DT(cint I, cint O) const;
  
};

tracedg tracedg::del(int e) const {

  int i,j,k,l,y,ncn[nl*ncr],nln[nl],*x[nl],*p=ncn;
  for(l=i=0;i<nl;++i)
  { x[l]=p;
    for(k=j=0;j<len[i];++j)
    { y=conn[i*ncr+j];
      if(y-e) p[k++]=y>e?y-1:y;
    }
    if(nln[l]=k) { ++l; p+=k; }
    // delete empty loops
  }
  ret tracedg(ncr-1,l,nln,(const int**)x);
}

tracedg tracedg::rtwist(int e=0) const {
  // this makes a reverse twist at edge e: e is split
  // into two edges e and ncr+1 and they are
  // joined by a valence two loop nl+1

  if(!e) e=ncr;
  int nc=ncr+1,nnl=nl+1,i,l,k,q,ncn[ncr],nln[nnl],*x[nnl],t[2]={e,nc};
  for(q=i=0;i<nl;++i)
  { if((k=member(e,x[i]=conn+i*ncr,nln[i]=len[i])) && q++) 
    {
      ERROR(nln[i]>ncr);
      copyarray(x[i],nln[i],ncn);
      (x[i]=ncn)[k-1]=nc;
    }
  }
  ERROR(q-2);
  ERROR(i-nl);
  nln[i]=2;
  x[i]=t;

  ret tracedg(nc,nnl,nln,(cint**)x);
}

  
tracedg tracedg::mult(int e, int mult) const {

  // it makes edge e into mult parallel ones, by keeping e
  // and adding edges with labels ncr+1,...,ncr+mult-1

  int i,j,k,l,y,o=0, nncr=ncr+mult-1,
      // nncr = new nr of crsgs
      ncn[nl*nncr],nln[nl],*x[nl],*p=ncn;

  for(i=0;i<nl;++i)
  { x[i]=p;
    for(k=j=0;j<len[i];++j)
    { if((p[k++]=conn[i*ncr+j])==e) {
	// since new edges must be placed on 
	// neighboring loops in reverse order to keep orientations
	// coherent, in o we record if we already passed one
	// loop connected by e or not
        if(o++)
	{ --k; for(l=nncr; l>ncr; p[k++]=l--); p[k++]=e; }
	else
	{ for(l=ncr; l<nncr; p[k++]=++l); }
      }
    }
    nln[i]=k;
    p+=k;
  }
  ret tracedg(nncr,nl,nln,(const int**)x);
}


tracedg twist(const tracedg & td, int *tvec)
{
  // tvec should have length==#crsgs in td, and the
  // result should be that crsg #i in td is replaced by
  // tvec[i-1] parallel copies

  // note this in order not to mix up the labels of the
  // crsgs one must 1st perform all non-deletions twistings
  // and then all deletions, with the deleted edges in descending order

  tracedg g=td;
  int i,n=td.ncr;
  for(i=0;i<n;++i) if(tvec[i]>1) g=g.mult(i+1,tvec[i]);
  // TRC(g);
  for(i=n;i--;) if(!tvec[i]) {
    g=g.del(i+1);
    // TRC(i+1);
    // TRC(g);
  }
  ret g;
}

int tracedg::nrcomp () const {

// this counts the nr of components of the link dg
// of which *this is the A-state

  int n2=ncr+2, s,i,d,j,l,e,t,y, np[ncr][2],q[ncr][2];
 
  for(j=0;j<ncr;++j) for(i=0;i<2;++i) np[j][i]=q[j][i]=0;
  // np[i][] records the loops joined by crsg trace #i+1
  // (keep in mind we # crsg traces from 1)
  // q[] is a dummy which crsgs have been traced
 
  for(j=0;j<nl;++j) for(i=0;i<len[j];++i)
  { e=conn[j*ncr+i];
    np[e-1][np[e-1][0]?1:0]=j;
  }
 
  // now q[i][j] indicates if component traced that
  // goes from crsg j+1 in direction i?1:-1 along
  // lower marked loop conn'd by crsg trace of j

  int dir,s0,nrcmp=0;
  for(j=0;j<ncr;j++) for(i=0;i<2;i++) if(q[j][i]==0)
  {
    ++nrcmp;
    t=j+1; 
    dir=d=i+i-1;
    s=s0=(np[j][0]<np[j][1]?0:1);
    // you alw start tracing a component from the
    // lower nrumered loop of the crsg you start with

    do {
 
      // TRC(t);
      q[t-1][dir<0?0:1]=1;
      l=np[t-1][s];
      y=member(t,conn+l*ncr,len[l]);
      ERROR(!y);
      if(dir<0) y+=len[l]-2;
      y=y%len[l];
      t=conn[l*ncr+y];
      ERROR(t<1 || t>ncr);
 
      // if you get both basept of a trace on the same loop
      // then remove them and set the inaq crsg counter nnq up
      s= (np[t-1][0]==l?1:0);
      ERROR(np[t-1][1-s]-l);
      dir*=-1;

    } while(t-j-1 || s-s0 || dir-d);
 
  }  

  ret nrcmp;
}

void tracedg::DT(cint I, cint O) const {

// this outputs the DT code of *this, assuming that L has
// I loops inside and thay are the first I, and O loops attached
// outside not on outer cycle, and they come before L, while
// all other outer cycle cloops come after L. So loops #0...I-1
// are inside L, and loops I...I+O-1 are attached outside L,
// while L is loop #I+O

// of which *this is the A-state

  static int cnt=0;
  int n2=ncr*2, s,i,j,l,e,t,y, np[ncr][2],gauss[n2],gl,bl[ncr],*u;
  // gauss[] stores the gauss code, and gl counts how many entries
  // in gauss[] were set
  const int nL=I+O;
  // ERROR(I+O-nL);
  // this is the index of L in the loops array of *this
 
  for(j=0;j<ncr;++j) for(i=0;i<2;++i) np[j][i]=0;
  // np[i][] records the loops joined by crsg trace #i+1
  // (keep in mind we # crsg traces from 1)
  // q[] is a dummy which crsgs have been traced
 
  for(j=0;j<nl;++j) for(i=0;i<len[j];++i)
  { e=conn[j*ncr+i]-1;
    np[e][np[e][0]?1:0]=j;
  }

  for(j=0;j<ncr;++j) {
    u=np[j];
    // bl indicates of we are iside or outside L
    // fills bl[] that indicates if crsg trace is inside L (bl[j]=-1)
    // or ouside (bl[j]=1)

    if(u[0]<I && u[1]<I) {
      ERROR(max(u[0],u[1])>1);
      // if two loops inside L are joined by a trace, then
      // this can be only the other two loos of the triangle,
      // and they are the 1st two
      bl[j]=-1; cont;
    }

    if(u[0]>nL && u[1]>nL) { bl[j]=1; cont; }
    // this is a crsg outside L betw other outer cycle loops

    ERROR(u[i=0]-nL && u[i=1]-nL);
    // all other crsgs must be attached to L

    bl[j]=u[1-i]<I ? -1 : 1;
  } 
  // now q[i][j] indicates if component traced that
  // goes from crsg j+1 in direction i?1:-1 along
  // lower marked loop conn'd by crsg trace of j

  int dir=1,s0;

  t=1;
  u=np[0];
  s=s0=(u[0]<u[1]?0:1);
  gl=0;
  // you alw start tracing a component from the
  // lower numbered loop of the crsg #1 you start with

  // TRC(I);
  // TRC(O);
  do {
 
    // TRC(t);
    // q[t-1][dir<0?0:1]=1;
    l=u[s];
    y=member(t,conn+l*ncr,len[l]);
    ERROR(!y);
    if(dir<0) y+=len[l]-2;
    y=y%len[l];
    t=conn[l*ncr+y];
    ERROR(t<1 || t>ncr);
 
    // if you get both basept of a trace on the same loop
    // then remove them and set the inaq crsg counter nnq up
    u=np[t-1];
    ERROR(u[0]<I && u[1]<I && max(u[0],u[1])>1);
    // if two loops inside L are joined by a trace, then
    // this can be only the other two loos of the triangle,
    // and they are the 1st two
    s= (u[0]==l?1:0);
    ERROR(u[1-s]-l);
    dir*=-1;
    ERROR(gl>=n2);
    gauss[gl++]=dir*bl[t-1]*t;

  } while(t-1 || s-s0 || dir-1);
 
  // TRC(gl);
  if(gl<n2) ret; // no knot

  cout<<ncr<<" "<<++cnt;
  for(i=1;i<n2;i+=2)
  { for(j=0; j<n2 && abs(gauss[j])-abs(gauss[i]);j+=2);
    ERROR(j>n2);
    cout<<" "<<sgn(gauss[j])*(j?j:n2);
  }
  cout<<endl;
  ret; 
}

tracedg tracedg::AtoB (int &nnq) const {

// this transforms *this into the B state loop dg
// assuming cyclic order in the loop arrays of *this
// is coherent; if B state is inadequate, all inaq
// loops are removed, and the nr of such loops is
// recorded in nnq (if B state is adequate, then nnq==0)

  // TRC(1);
  int n2=ncr+2, s,i,w,j,k,l,e,t,y,
      np[ncr][2],ncn[n2*ncr],nlen[n2],*x[n2],q[ncr][2],na[ncr];
 
  for(j=0;j<ncr;++j) for(i=0;i<2;++i) np[j][i]=q[j][i]=0;
  // np[i][] records the loops joined by crsg trace #i+1
  // (keep in mind we # crsg traces from 1)
  // q[] is a dummy which crsgs have been traces
 
  // nnq counts nr of inaq crsgs in B state
  // na[] collects B-inaq crsgs for internal renumbering
 
  for(j=0;j<nl;++j) for(i=0;i<len[j];++i)
  { e=conn[j*ncr+i];
    np[e-1][np[e-1][0]?1:0]=j;
  }
 
  int *p=ncn,lcnt=0;
  for(nnq=lcnt=j=0;j<ncr;j++) for(i=0;i<2;i++) if(q[j][i]==0)
  {
    x[lcnt]=p;
    int &u=nlen[lcnt];
    u=0;
    t=j+1;
    s=i;

    do {
 
      // TRC(t);
      // TRC(s);
      q[t-1][s]=1;
      l=np[t-1][s];
      y=member(t,conn+l*ncr,len[l]);
      ERROR(!y);
      y=y%len[l];
      t=conn[l*ncr+y];
      ERROR(t<1 || t>ncr);
 
      // if you get both basept of a trace on the same loop
      // then remove them and set the inaq crsg counter nnq up
      if(w=member(t,p,u)) {
        na[nnq++]=t;
        copyarray(p+w,u-w,p+w-1);
        // this call is unclean, but makes use of the internal
        // knowledge that copyarray() copies the arrays forwardly
        --u;
      }
      else
      // no double trace
      p[u++]=t;
 
      s= (np[t-1][0]==l?1:0);
      ERROR(np[t-1][1-s]-l);

    } while(t-j-1 || s-i);
 
    p+=u;
    ++lcnt;
  }  
  // TRC(1);

  if(nnq) {
    // exist inaq crsgs in B state;
    // now must renumber crsgs to discard inaq ones

    qsort(na,nnq);
    // TRCA(na,nnq);

    int nnr[ncr+1];
    // nnr[i] is the new nr of crsg #i
    
    for(k=0,i=1;i<=ncr;++i)
    {
      while(k<nnq && i==na[k]) nnr[i++]=0,++k;
      nnr[i]=i-k;
      // TRC(i);
      // TRC(k);
      ERROR(!nnr[i]);
    }
    // TRCA(nnr+1,i);

    for(i=0;i<lcnt;++i) for(j=0;j<nlen[i];++j)
    { 
      x[i][j]=nnr[x[i][j]];
      ERROR(!x[i][j]);
    }
  }
  // TRC(1);
  ret tracedg(ncr-nnq,lcnt,nlen,(const int**)x);
}

// bool saqchk(int cr, int *dow, int sg, tracedg & tr, int &wr) 
// got obsolete and deleted

int saqchktr(const tracedg & tr, int &xa, int &ntri,
  int &xig, int &al, int &anr) 
// is dow sg-semiadeq (+=A, -=B)
// xa = xhi(A(D)), ntri=nr of triangles, xig=chi(IG(A))
// al = # loops in A(D), anr = a_A(D) (atom nr of A(D))
{
  // TRC(1);
  int i,x,j,dr,k,ee,e,e2,cr=tr.ncr,ns=tr.nl,cr2=2*cr,other_lop,
       r,y,nnr[ns][cr],nr[cr],w[cr2];

  // TRC(2);
  al=ns;

  // TRC(3);
  int cn[ns][ns],connchg[ns][ns],vrt[ns],ed[2*cr],np[cr][2];
  for(j=0;j<ns;j++) for(i=0;i<ns;i++)
    cn[i][j]= connchg[i][j]=0;
  for(j=0;j<cr;j++) for(i=0;i<2;i++) np[j][i]=0;

  // TRC(tr);
  for(j=0;j<ns;++j) { 
    nr[j]=tr.len[j];
    for(i=0;i<nr[j];++i) {
      e=nnr[j][i]=tr.conn[j*cr+i];
      for(k=0;k<2 && np[e-1][k];++k);
      if(k<2) np[e-1][k]=j;
      // NOTE THAT THIS WORKS WELL EVEN IF WE WORK HERE WITH
      // LOOPS j NUMBERED FROM 0
    }
  }
  // TRC(tr);
  // for(i=0;i<cr;++i) TRCA(np[i],2);
  for(j=0;j<ns;++j) {
    nr[j]=tr.len[j];
    for(i=0;i<nr[j];++i) {
      e=nnr[j][i];
      other_lop=np[e-1][np[e-1][0]==j?1:0];
      if(other_lop>j) {
	++cn[other_lop][j];
	++cn[j][other_lop];
      } 
      // copyarray(tr.conn+j*cr, nr[j]=tr.len[j], nnr[j]); 
    }
  }
  // TRC(ns);

  for(e=ee=j=0;j<ns;j++) for(i=vrt[j]=j+1;i<ns;i++) {
     if(cn[i][j]) {
       ed[2*e]=j+1;
       ed[2*e+1]=i+1;
       ++e;
       if(cn[i][j]>1) ++ee; 
     }
  }
  // TRC(ns);

  // TRC(graph(ns,e,vrt,ed).atomnr());
  anr=graph(ns,e,vrt,ed).atomnr();
  // counts mult edges in ee and reduces y from #verts to
  // Euler char of G(A)
  // TRC(anr);

  // TRC(ns);
  // TRC(xa);
  // TRC(y);
  // TRC(y);
  xa=ns-e;
  // Euler char of G(A)

  // this counts the triangles
  for(y=j=0;j<ns;j++) for(i=j+1;i<ns;i++) if(cn[i][j]) {
    for(k=i+1;k<ns;k++)  if(cn[k][j] && cn[i][k]) y++;
  }

  // TRC(y);
  ntri=y;
  
  // TRCA(nr,ns);
  // TRC(tr);
  for(j=0;j<ns;j++) {
    r=nr[j];
    // TRC(j);
    // TRC(r);
    if(r<4) cont;
    int x[cr],*olop=nnr[j],crs,l,m;
    // TRC(j);

    // olop[] records the ther loops j is conn'd to
    // by tre traces in cyclic order

    for(k=0;k<ns;k++) for(i=0;i<ns;i++) connchg[k][i]=0;

    // TRCA(olop,r);
    for(i=0;i<r;++i) {
      e=olop[i];
      m=np[e-1][np[e-1][0]==j?1:0];
      zeroarray(x,cr);
      // x[] counts for what edges l neightbored to j &diff from m
      // the switch of connection betw traces of l and m on j
      // has been accounted for
      for(k=(i+1)%r;
          e2=olop[k],(l=np[e2-1][np[e2-1][0]==j?1:0])-m;
	  k=(k+1)%r
	 )
      {
        if(x[l]==0) {
           x[l]= ++connchg[l][m]; ++connchg[m][l];
        }
        // TRC(k);
      } // for k
    } // for i
    // TRC(j);
 
    for(k=ns-1;k>=0;--k) for(i=k-1;i>=0;--i)
    {
      if(connchg[k][i]%2 // && connchg[k][i]>0
	 ) { 
        // TRC(i);
        // TRC(k);
	// TRCA(olop,r);
	// TRC(connchg[k][i]);
        cerr<<"error!!!"<<endl; 
      }
      if(connchg[k][i]>2) ee--;
    }
  } // for j
 
  // TRC(ee);
  xig=ee;
  ret true;
}

// this now tests and prints twist vectors whose dgs
// match all invs we can calculate

int four, five, six;

// five is the twist vector entry for non-trivial
// second twist ; six is needed for technical reasons

inline int GROS(int *x,int j)
{ int i; for(i=0;i<j && x[i]-five && x[i]-four;++i); ret i-j; }

inline bool GROS(int **x,int j)
{ int i; for(i=0;i<j && *x[i]-five && *x[i]-four;++i); ret i<j; }

int sum(const int *X, int len)
{ int i,s;
  for (i=s=0;i<len;s+=X[i++]);
  ret s;
}

// void test(const tracedg & t, int *X, int ln)
// {
//    ERROR(ln-t.ncr);
//    int nq;
//    // TRC(twist(t,X));
//    tracedg td1=twist(t,X),td2=td1.AtoB(nq),td[2]={td1,td2};
//    // TRC(td1);
//    // TRC(td2);
//    // ERROR(nq==0 && td1.nrcomp()-td2.nrcomp());
//    if(nq || td1.nrcomp()>1) ret;
//    int j, xa, ntri, xig, al, an;
// 
//    for(j=0;j<=1;j++) {
//      saqchktr(td[j],xa,ntri,xig,al,an);
//      if(xa+1 || ntri-1 || xig+1 || an-4) 
//        // test for the specific data of our knots
//        // NOTE THE CHANGED VALUES IN COMPARISON TO
//        // ../V12test/aqv23tst7.C
//      {
//        break;
//      }
//    }
//    if(j==2) TRCA(X,ln);
// }

void test(const tracedg & t, int *X, cint ln, cint I, cint O)
{
   ERROR(ln-t.ncr);
   // I - # of loops inside L,
   // O - # of loops of depth 0 outside outer cycle outside L
   int nq,s=sum(X,ln);
   // TRC(t);
   // TRC(sum(X,ln));
   // TRC(t.rtwist());
   tracedg td0=s%2 ? t.rtwist() : t,
   // adjust outer cycle twisting to have even nr of crsgs
           td1=twist(td0,X),td2=td1.AtoB(nq),td[2]={td1,td2};
   // TRC(td1);
   // TRC(td2);
   // ERROR(nq==0 && td1.nrcomp()-td2.nrcomp());
   if(nq || td1.nrcomp()>1) ret;
   int nc=td1.ncr, j, xa, ntri, xig, al[2], an, x, y, z;

   for(j=ln;j<ln+7;++j) ERROR(X[j]-1);
   ERROR(nc%2);
   // TRCA(X,ln+s%2);
   // TRC(td0);
   // TRC(td1);
   // td1.DT(I,O);

   for(j=0;j<=1;j++) {
     saqchktr(td[j],xa,ntri,xig,al[j],an);
     // cout<<"AB"[j]<<"-state: "<<"ntri ="<<ntri<<" xig="<<
     //   xig<<" #lp="<<al[j]<<" atoms="<<an<<endl;
     if(xa+1 || ntri-1 || xig+1 || an-4) 
       // test for the specific data of our knots
       // NOTE THE CHANGED VALUES IN COMPARISON TO
       // ../V12test/aqv23tst7.C
     {
       ret;
     }
   }

   // gm must be 2 or 4 for all cands
   if(al[0]+al[1]==nc) ret;
   ERROR(al[0]+al[1]-nc+2)

   x=al[1]-al[0]; // #B-loops - #A-loops
   z=GROS(X,ln)+ln;
   if(z==ln) {
     // cant adjust by changing mult of a leg
     // so dg belongs to a sproadic series in which I can only
     // enlarge the out cycle by adding pairs of simple egdes
     // (reverse twisting)
     if(x<0 ||
     // if B-state has too few loops, then cannot made loops
     // equal by reverse twisting at outer cycle, so out
        (sum(X,ln)+x)%4) ret;
     // do not get dg of crsg nr mult of 4 after twists

     // otherwise can get one dg, after x/2 full reverse twists at
     // outer cycle
     cout<<s<<" {";
     for(j=0;j<ln;++j) { cout<<X[j]<<(j<ln-1?",":"}"); }
     if(x) {
       cout<<"+"<<x/2<<" twists@outer cycle"<<endl;
       // if at most 3 full reverse twists (x<=6), then X is long
       // enough to give twist vector for reverse twisted cycle dg, so
       // output the DT code
       if(x<7) { 
	 while(x--) td0=td0.rtwist();
         twist(td0,X).DT(I,O);
       }
     } else
     { 
       cout<<endl;
       td1.DT(I,O);
     }
     ret;
   }
   y=X[z];
   X[z]-=x; //corrects the twisting so that A & B-state have same #lops
   if(sum(X,ln)%4) { X[z]=y; ret; }
   // with corrected twisting crsg # must be div by 4 (need to rule
   // out dgs of 20,24,28,... crsgs)
   cout<<s<<" {";
   for(j=0;j<ln;++j) {
     cout<<X[j]<<( x && X[z]>=0 && X[z]<=2 && j==z ? "*" : "") <<
       (j<ln-1?",":"}");
   }
   cout<<endl;

   // output orig dg and the next three (each nect dg is obtained by
   // one reverse twist at outer cycle and changing the meg mult by
   // 2
   for(j=0;j<4;++j) {
     (x || j ? twist(td0,X) : td1).DT(I,O);
     if(j<3) { X[z]+=2; td0=td0.rtwist().rtwist(); }
   }
   X[z]=y;
}

//
// this is now from V12test/v_ktest4.C 
//



int setvalues_pair (int *x, int restr, int modus)
{
  // generates next admissible vector for valence 2-cicrle
  // attached at cycle loop;
  // valence 2-cicrle has outer edges a,b

  // x is int[2]
  // 1st time must be called with modus=0, then with
  // modus>0;
  // restr is 0 iff in previous conf a cycle in B'(D)
  // has not yet been encountered (i.e. if one can
  // still create a cycle now)

  // ret value: -1 if all admissible settings exhausted,
  // else generate next setting, and ret 0 if no restr,
  // or 1 if restr (B'(D) has encountered cycle)

  int j;
  const int vec[5][2]={{1,1},{1,2},{1,4},{1,5},{2,5}};

  if(!modus) { j=0 ; } else {
    for(j=0;j<5 && diffarray(x,vec[j],2);++j);  ++j;
    if(j>4) ret -1;
  }

  copyarray(vec[j],2,x);
  ret restr;
}

// the foll procs generate multiplicities of outer cycle
// edges 

int setvalues_cycle_A (int *a, int restr, int modus)
// here w is int[12]
{ 
  int &f=a[0], &g=a[1], &h=a[2], &I=a[3], &k=a[4], &l=a[5], &m=a[6],
      &n=a[7], &w=a[8], &x=a[9], &y=a[10], &z=a[11], j, i, u, v;

  //          f g h i k l m n w x y z
  int mn[12]={2,0,0,0,0,0,0,0,1,1,1,1},
      mx[10]={2,5,5,5,5,5,5,5,5,5};
      // f=2,y=z=1 are never moved

  if (!modus) { for(i=0;i<12;++i) a[i]=mn[i]; }

  for(;;) {
    if(modus) {
      for(j=9;j>0 && 
	(a[j]=a[j]==2?four:a[j]+1)>=
	// we admit only alues 1,2,five for a[j]
	( restr || GROS(a+1,j-1) ? 3 : mx[j]+1 ) ;
	a[j]= mn[j], --j 
      );
      if(j<=0) ret -1;
    }

    if((v=GROS(a+1,9)) && restr 
       // if restr no >=3 is ok
       || (u=member(0,a+1,7)) && (x>1 || w>1 || member(0,a+1+u,7-u))
       // two of g,i,h,k,l,m,n are 0, or one is 0 and (x>1 || w>1)
       || x>1 && w>1
      ) 
      // inconsistent or redundant

      modus = 1;

      else ret max ( restr , v ? 1 : 0 );
  } // for
}

int setvalues_inner(int *w, int restr, int modus)
// w is int[3]; 
{ 
  int &a=w[0], &b=w[1], &c=w[2], j, i;

  if (!modus) { for(i=0;i<3;++i) w[i]=1; }

  for(;;) {
    if(modus) {
      for(j=2;j>=0 && 
	(w[j]=w[j]==2?four:w[j]+1)>=
	// we admit only alues 1,2,five for w[j]
	( restr || GROS(w,j) ? 3 : six ) ;
	w[j--]= 1 
      );
      if(j<0) ret -1;
    }

    if(restr && GROS(w,3) || // if restr no >=3 is ok
      c-2||               // this is the inaq edge
      a%2+b%2==0        // disconnected (c is alw even)
    ) 
    // inconsistent or redundant

      modus = 1;

      else ret max ( restr , GROS(w,3) ? 1 : 0 );
  } // for
}


// now come admissible twist vector generation routines

void conf_A(const tracedg & k)
{
  // O-12, A-2, B-3 + 7 extra crsgs for twisted dgs in test()
  // total X-17 + 7 = 24
  // TRC(k);
  if(k.ncr-17 && k.ncr)
    { cerr<<"wrong crsg nr in conf_B13"<<endl; ret; }

  // the cases in which to two cycle loops inside sth is
  // attached
  int X[24], *A=X, *B=A+2,  *O=B+3, i, q, o, r, c=0;
   
  for(i=17;i<24;++i) X[i]=1;
  for(o=0; o=setvalues_cycle_A(O, 0 ,o)+1;) 
    for(q=0; q=setvalues_pair(A,o-1,q)+1;) 
      for(r=0; r=setvalues_inner(B,q-1,r)+1;)
	  { if(++c%10000==0) TRCA(X,17);
            test(k,X,17,2,0); 
	  }
}

main(int argc, char *argv[]) { 

  tracedg t1;
  ifstream f1("trdg_A.in",ios::in);

  f1>>t1;

  for(five=5;five<6;++five)
  {
    six=five+1;
    four=five-1;
    cout<<"A"<<endl;
    conf_A(t1);
  }
}

